@@ -5,7 +5,10 @@ import os
 import md4c
 import polib
 
-from mdpo.command import parse_mdpo_html_command
+from mdpo.command import (
+    normalize_mdpo_command_aliases,
+    parse_mdpo_html_command,
+)
 from mdpo.io import filter_paths, to_glob_or_content
 from mdpo.md4c import DEFAULT_MD4C_GENERIC_PARSER_EXTENSIONS
 from mdpo.po import find_entry_in_entries, po_escaped_string
@@ -14,6 +17,74 @@ from mdpo.text import min_not_max_chars_in_a_row
 
 
 class Md2Po:
+    __slots__ = (
+        'filepaths',
+        'content',
+        'pofile',
+        'msgstr',
+        'found_entries',
+        'disabled_entries',
+        'wrapwidth',
+        'ignore_msgids',
+        'command_aliases',
+        'mark_not_found_as_obsolete',
+        'extensions',
+        'plaintext',
+        'include_codeblocks',
+        'xheaders',
+
+        '_current_msgid',
+        '_current_tcomment',
+        '_current_msgctxt',
+
+        '_disable',
+        '_disable_next_line',
+        '_enable_next_line',
+        '_include_next_codeblock',
+        '_include_xheaders',
+
+        '_enterspan_replacer',
+        '_leavespan_replacer',
+
+        'bold_start_string',
+        'bold_start_string_escaped',
+        'bold_end_string',
+        'bold_end_string_escaped',
+        'italic_start_string',
+        'italic_start_string_escaped',
+        'italic_end_string',
+        'italic_end_string_escaped',
+        'code_start_string',
+        'code_start_string_escaped',
+        'code_end_string',
+        'code_end_string_escaped',
+        'link_start_string',
+        'link_end_string',
+        'strikethrough_start_string',
+        'strikethrough_end_string',
+        'latexmath_start_string',
+        'latexmath_end_string',
+        'latexmathdisplay_start_string',
+        'latexmathdisplay_end_string',
+        'wikilink_start_string',
+        'wikilink_end_string',
+        'underline_start_string',
+        'underline_end_string',
+
+        '_inside_uspan',
+        '_inside_htmlblock',
+        '_inside_codeblock',
+        '_inside_pblock',
+        '_inside_liblock',
+        '_inside_codespan',
+        '_codespan_start_index',
+        '_codespan_backticks',
+        '_current_aspan_href',
+        '_current_wikilink_target',
+        '_current_imgspan',
+        '_uls_deep',
+    )
+
     def __init__(self, glob_or_content, **kwargs):
         is_glob, glob_or_content = to_glob_or_content(glob_or_content)
         if is_glob:
@@ -27,11 +98,16 @@ class Md2Po:
         self.pofile = None
         self.msgstr = kwargs.get('msgstr', '')
         self.found_entries = []
+        self.disabled_entries = []
         self._current_msgid = ''
         self._current_tcomment = None
         self._current_msgctxt = None
 
         self.wrapwidth = kwargs.get('wrapwidth', 78)
+        self.ignore_msgids = kwargs.get('ignore_msgids', [])
+        self.command_aliases = normalize_mdpo_command_aliases(
+            kwargs.get('command_aliases', {}),
+        )
 
         self.mark_not_found_as_obsolete = kwargs.get(
             'mark_not_found_as_obsolete', True,
@@ -49,10 +125,11 @@ class Md2Po:
         self._disable = False
         self._disable_next_line = False
         self._enable_next_line = False
-        self.disabled_entries = []
 
         self._include_next_codeblock = False
-        self._include_xheaders = False
+
+        self.xheaders = None
+        _include_xheaders = False
 
         if not self.plaintext:
             self.bold_start_string = kwargs.get('bold_start_string', '**')
@@ -75,8 +152,6 @@ class Md2Po:
                 self.italic_end_string,
             )
 
-            self._bold_italic_context = False
-
             # codespans are built by a indetermined number of 'x' characters
             # so we take only the first
             self.code_start_string = kwargs.get('code_start_string', '`')[0]
@@ -92,8 +167,8 @@ class Md2Po:
             self.link_start_string = kwargs.get('link_start_string', '[')
             self.link_end_string = kwargs.get('link_end_string', ']')
 
-            self._include_xheaders = kwargs.get('xheaders', False)
-            if self._include_xheaders:
+            _include_xheaders = kwargs.get('xheaders', False)
+            if _include_xheaders:
                 self.xheaders = {
                     'x-mdpo-bold-start': self.bold_start_string,
                     'x-mdpo-bold-end': self.bold_end_string,
@@ -132,7 +207,7 @@ class Md2Po:
                 self._leavespan_replacer[md4c.SpanType.DEL] = \
                     self.strikethrough_end_string
 
-                if self._include_xheaders:
+                if _include_xheaders:
                     self.xheaders.update({
                         'x-mdpo-strikethrough-start':
                             self.strikethrough_start_string,
@@ -165,7 +240,7 @@ class Md2Po:
                 self._leavespan_replacer[md4c.SpanType.LATEXMATH_DISPLAY] = \
                     self.latexmathdisplay_end_string
 
-                if self._include_xheaders:
+                if _include_xheaders:
                     self.xheaders.update({
                         'x-mdpo-latexmath-start': self.latexmath_start_string,
                         'x-mdpo-latexmath-end': self.latexmath_end_string,
@@ -188,7 +263,7 @@ class Md2Po:
                 self._leavespan_replacer[md4c.SpanType.WIKILINK] = \
                     self.wikilink_end_string
 
-                if self._include_xheaders:
+                if _include_xheaders:
                     self.xheaders.update({
                         'x-mdpo-wikilink-start': self.wikilink_start_string,
                         'x-mdpo-wikilink-end': self.wikilink_end_string,
@@ -208,7 +283,7 @@ class Md2Po:
                 self._leavespan_replacer[md4c.SpanType.U] = \
                     self.underline_end_string
 
-                if self._include_xheaders:
+                if _include_xheaders:
                     self.xheaders.update({
                         'x-mdpo-underline-start': self.underline_start_string,
                         'x-mdpo-underline-end': self.underline_end_string,
@@ -236,6 +311,8 @@ class Md2Po:
         self._uls_deep = 0
 
     def _save_msgid(self, msgid, tcomment=None, msgctxt=None):
+        if msgid in self.ignore_msgids:
+            return
         entry = polib.POEntry(
             msgid=msgid,
             msgstr=self.msgstr,
@@ -243,7 +320,8 @@ class Md2Po:
             msgctxt=msgctxt,
         )
         _equal_entry = find_entry_in_entries(
-            entry, self.pofile,
+            entry,
+            self.pofile,
             compare_obsolete=False,
             compare_msgstr=False,
         )
@@ -280,43 +358,48 @@ class Md2Po:
         self._current_msgctxt = None
 
     def _process_command(self, text):
-        command, comment = parse_mdpo_html_command(text)
-        if command is None:
+        original_command, comment = parse_mdpo_html_command(text)
+        if original_command is None:
             return
 
-        if command == 'disable-next-line':
+        try:
+            command = self.command_aliases[original_command]
+        except KeyError:  # not custom command
+            command = original_command
+
+        if command == 'mdpo-disable-next-line':
             self._disable_next_line = True
-        elif command == 'disable':
+        elif command == 'mdpo-disable':
             self._disable = True
-        elif command == 'enable':
+        elif command == 'mdpo-enable':
             self._disable = False
-        elif command == 'enable-next-line':
+        elif command == 'mdpo-enable-next-line':
             self._enable_next_line = True
-        elif command == 'translator':
+        elif command == 'mdpo-translator':
             if not comment:
                 raise ValueError(
                     'You need to specify a string for the'
                     ' extracted comment with the command'
-                    ' \'mdpo-translator\'.',
+                    f' \'{original_command}\'.',
                 )
-            self._current_tcomment = comment.strip(' ')
-        elif command == 'context':
+            self._current_tcomment = comment.rstrip()
+        elif command == 'mdpo-context':
             if not comment:
                 raise ValueError(
                     'You need to specify a string for the'
-                    ' context with the command \'mdpo-context\'.',
+                    f' context with the command \'{original_command}\'.',
                 )
-            self._current_msgctxt = comment.strip(' ')
-        elif command == 'include-codeblock':
+            self._current_msgctxt = comment.rstrip()
+        elif command == 'mdpo-include-codeblock':
             self._include_next_codeblock = True
-        elif command == 'include':
+        elif command == 'mdpo-include':
             if not comment:
                 raise ValueError(
                     'You need to specify a message for the'
                     ' comment to include with the command'
-                    ' \'mdpo-include\'.',
+                    f' \'{original_command}\'.',
                 )
-            self._current_msgid = comment.strip(' ')
+            self._current_msgid = comment.rstrip()
             self._save_current_msgid()
 
     def enter_block(self, block, details):
@@ -548,7 +631,7 @@ class Md2Po:
                     else:
                         entry.obsolete = False
 
-        if self._include_xheaders:
+        if self.xheaders:
             self.pofile.metadata.update(self.xheaders)
 
         if save and _po_filepath:
@@ -559,11 +642,23 @@ class Md2Po:
 
 
 def markdown_to_pofile(
-    glob_or_content, ignore=[], msgstr='', po_filepath=None, save=False,
-    mo_filepath=None, plaintext=False, wrapwidth=78,
+    glob_or_content,
+    ignore=[],
+    msgstr='',
+    po_filepath=None,
+    save=False,
+    mo_filepath=None,
+    plaintext=False,
+    wrapwidth=78,
     mark_not_found_as_obsolete=True,
-    extensions=DEFAULT_MD4C_GENERIC_PARSER_EXTENSIONS, po_encoding=None,
-    md_encoding='utf-8', xheaders=False, include_codeblocks=False, **kwargs,
+    extensions=DEFAULT_MD4C_GENERIC_PARSER_EXTENSIONS,
+    po_encoding=None,
+    md_encoding='utf-8',
+    xheaders=False,
+    include_codeblocks=False,
+    ignore_msgids=[],
+    command_aliases={},
+    **kwargs,
 ):
     """Extracts all the msgids from a string of Markdown content or a group of
     files.
@@ -613,6 +708,13 @@ def markdown_to_pofile(
             result. This is useful if you want to translate all your blocks
             of code. Equivalent to append ``<!-- mdpo-include-codeblock -->``
             command before each code block.
+        ignore_msgids (list): List of msgids ot ignore from being extracted.
+        command_aliases (dict): Mapping of aliases to use custom mdpo command
+            names in comments. The ``mdpo-`` prefix in command names resolution
+            is optional. For example, if you want to use ``<!-- mdpo-on -->``
+            instead of ``<!-- mdpo-enable -->``, you can pass the dictionaries
+            ``{"mdpo-on": "mdpo-enable"}`` or ``{"mdpo-on": "enable"}`` to this
+            parameter.
 
     Examples:
         >>> content = 'Some text with `inline code`'
@@ -630,12 +732,22 @@ def markdown_to_pofile(
         :class:`polib.POFile` Pofile instance with new msgids included.
     """
     return Md2Po(
-        glob_or_content, ignore=ignore, msgstr=msgstr, plaintext=plaintext,
+        glob_or_content,
+        ignore=ignore,
+        msgstr=msgstr,
+        plaintext=plaintext,
         wrapwidth=wrapwidth,
         mark_not_found_as_obsolete=mark_not_found_as_obsolete,
-        extensions=extensions, xheaders=xheaders,
-        include_codeblocks=include_codeblocks, **kwargs,
+        extensions=extensions,
+        xheaders=xheaders,
+        include_codeblocks=include_codeblocks,
+        ignore_msgids=ignore_msgids,
+        command_aliases=command_aliases,
+        **kwargs,
     ).extract(
-        po_filepath=po_filepath, save=save, mo_filepath=mo_filepath,
-        po_encoding=po_encoding, md_encoding=md_encoding,
+        po_filepath=po_filepath,
+        save=save,
+        mo_filepath=mo_filepath,
+        po_encoding=po_encoding,
+        md_encoding=md_encoding,
     )
