@@ -5,7 +5,12 @@
 import argparse
 import sys
 
-from mdpo.cli import add_common_cli_arguments, parse_list_cli_argument
+from mdpo.cli import (
+    add_common_cli_first_arguments,
+    add_common_cli_latest_arguments,
+    parse_command_aliases_cli_argument,
+    parse_list_cli_argument,
+)
 from mdpo.md2po import markdown_to_pofile
 from mdpo.md4c import DEFAULT_MD4C_GENERIC_PARSER_EXTENSIONS
 
@@ -18,7 +23,7 @@ DESCRIPTION = (
 
 def build_parser():
     parser = argparse.ArgumentParser(description=DESCRIPTION, add_help=False)
-    add_common_cli_arguments(parser)
+    add_common_cli_first_arguments(parser)
     parser.add_argument(
         'glob_or_content', metavar='GLOB_OR_CONTENT',
         nargs='*',
@@ -105,6 +110,12 @@ def build_parser():
              ' Equivalent to append \'<!-- mdpo-include-codeblock -->\''
              ' command before each code block.',
     )
+    parser.add_argument(
+        '--ignore-msgids', dest='ignore_msgids', default=None,
+        help='Path to a plain text file where all msgids to ignore from being'
+             ' extracted are located, separated by newlines.',
+    )
+    add_common_cli_latest_arguments(parser)
     return parser
 
 
@@ -122,6 +133,15 @@ def parse_options(args=[]):
     if opts.ignore:
         opts.ignore = parse_list_cli_argument(opts.ignore)
     opts.extensions = opts.extensions.split(',')
+    if opts.ignore_msgids is not None:
+        with open(opts.ignore_msgids) as f:
+            opts.ignore_msgids = f.read().splitlines()
+    else:
+        opts.ignore_msgids = []
+
+    opts.command_aliases = parse_command_aliases_cli_argument(
+        opts.command_aliases,
+    )
 
     return opts
 
@@ -141,6 +161,8 @@ def run(args=[]):
         md_encoding=opts.md_encoding,
         xheaders=opts.xheaders,
         include_codeblocks=opts.include_codeblocks,
+        ignore_msgids=opts.ignore_msgids,
+        command_aliases=opts.command_aliases,
     )
     if isinstance(opts.wrapwidth, int):
         kwargs['wrapwidth'] = opts.wrapwidth
