@@ -7,7 +7,10 @@ import textwrap
 import md4c
 import polib
 
-from mdpo.command import parse_mdpo_html_command
+from mdpo.command import (
+    normalize_mdpo_command_aliases,
+    parse_mdpo_html_command,
+)
 from mdpo.io import filter_paths, to_file_content_if_is_file
 from mdpo.md import (
     escape_links_titles,
@@ -17,10 +20,73 @@ from mdpo.md import (
 from mdpo.md4c import DEFAULT_MD4C_GENERIC_PARSER_EXTENSIONS
 from mdpo.po import po_escaped_string
 from mdpo.polib import *  # noqa
-from mdpo.text import min_not_max_chars_in_a_row
+from mdpo.text import (
+    min_not_max_chars_in_a_row,
+    wrap_different_first_line_width,
+)
 
 
 class Po2Md:
+    __slots__ = (
+        'pofiles',
+        'output',
+        'extensions',
+        'disabled_entries',
+        'translations',
+        'translations_with_msgctxt',
+        'command_aliases',
+        'wrapwidth',
+
+        'bold_start_string',
+        'bold_start_string_escaped',
+        'bold_end_string',
+        'bold_end_string_escaped',
+        'italic_start_string',
+        'italic_start_string_escaped',
+        'italic_end_string',
+        'italic_end_string_escaped',
+        'code_start_string',
+        'code_start_string_escaped',
+        'code_end_string',
+        'code_end_string_escaped',
+        'link_start_string',
+        'link_end_string',
+        'wikilink_start_string',
+        'wikilink_end_string',
+
+        # internal config
+        '_current_msgid',
+        '_current_msgctxt',
+        '_current_line',
+        '_outputlines',
+        '_disable_next_line',
+        '_disable',
+        '_enable_next_line',
+        '_enterspan_replacer',
+        '_leavespan_replacer',
+
+
+        # state
+        '_inside_htmlblock',
+        '_inside_codeblock',
+        '_inside_indented_codeblock',
+        '_inside_pblock',
+        '_inside_liblock',
+        '_inside_codespan',
+        '_codespan_start_index',
+        '_codespan_backticks',
+        '_codespan_inside_current_msgid',
+        '_inside_quoteblock',
+        '_current_aspan_href',
+        '_current_imgspan',
+        '_current_thead_aligns',
+        '_aimg_title_inside_current_msgid',
+        '_ul_marks',
+        '_ol_marks',
+        '_current_list_type',
+        '_current_wikilink_target',
+    )
+
     def __init__(self, pofiles, ignore=[], po_encoding=None, **kwargs):
         self.pofiles = [
             polib.pofile(pofilepath, encoding=po_encoding) for pofilepath in
@@ -45,6 +111,12 @@ class Po2Md:
         self.translations = None
         self.translations_with_msgctxt = None
 
+        self.command_aliases = normalize_mdpo_command_aliases(
+            kwargs.get('command_aliases', {}),
+        )
+
+        self.wrapwidth = kwargs.get('wrapwidth', 80)
+
         self.bold_start_string = kwargs.get('bold_start_string', '**')
         self.bold_start_string_escaped = po_escaped_string(
             self.bold_start_string,
@@ -65,8 +137,6 @@ class Po2Md:
             self.italic_end_string,
         )
 
-        self._bold_italic_context = False
-
         self.code_start_string = kwargs.get('code_start_string', '`')[0]
         self.code_start_string_escaped = po_escaped_string(
             self.code_start_string,
@@ -132,7 +202,11 @@ class Po2Md:
         self._ul_marks = []
 
         # [numerical iterm order, current delimitier] for OL blocks
-        self._current_ol_delimiters = []
+        self._ol_marks = []
+
+        # current lists type (list nesting): ['ul' or 'ol', [True, False]]
+        #   (second parameter is tasklist item or not)
+        self._current_list_type = []
 
         self._current_wikilink_target = None
 
@@ -141,16 +215,21 @@ class Po2Md:
         if command is None:
             return
 
-        if command == 'disable-next-line':
+        try:
+            command = self.command_aliases[command]
+        except KeyError:  # not custom command
+            pass
+
+        if command == 'mdpo-disable-next-line':
             self._disable_next_line = True
-        elif command == 'disable':
+        elif command == 'mdpo-disable':
             self._disable = True
-        elif command == 'enable':
+        elif command == 'mdpo-enable':
             self._disable = False
-        elif command == 'enable-next-line':
+        elif command == 'mdpo-enable-next-line':
             self._enable_next_line = True
-        elif command == 'context' and comment:
-            self._current_msgctxt = comment.strip(' ')
+        elif command == 'mdpo-context' and comment:
+            self._current_msgctxt = comment.rstrip()
 
     def _escape_translation(self, text):
         # escape '"' characters inside links and image titles
@@ -195,23 +274,50 @@ class Po2Md:
 
         if not self._inside_codeblock:
             translation = self._escape_translation(translation)
-
         elif self._inside_indented_codeblock:
             # add 4 spaces before each line including next indented block code
             translation = '    %s' % re.sub('\n', '\n    ', translation)
-        if self._inside_liblock:
-            translation = '\n'.join(textwrap.wrap(translation, width=79))
-        if self._inside_pblock:
+
+        if self._inside_liblock or self._inside_quoteblock:
+            first_line_width_diff = -2
+            if self._inside_liblock:
+                if self._current_list_type[-1][0] == 'ul':
+                    if self._current_list_type[-1][-1][-1] is True:
+                        first_line_width_diff = -6
+                else:
+                    first_line_width_diff = -3
+
+            if self._codespan_inside_current_msgid:
+                lines = fixwrap_codespans(
+                    translation.split('\n'),
+                    width=self.wrapwidth,
+                    first_line_width=self.wrapwidth + first_line_width_diff,
+                )
+            else:
+                lines = wrap_different_first_line_width(
+                    translation,
+                    width=self.wrapwidth,
+                    first_line_width_diff=first_line_width_diff,
+                    break_long_words=False,
+                )
+            translation = '\n'.join(lines)
+        elif self._inside_pblock:
             # wrap paragraphs fitting with markdownlint
             if self._codespan_inside_current_msgid:
                 # fix codespans wrapping
-                lines = fixwrap_codespans(translation.split('\n'))
+                lines = fixwrap_codespans(
+                    translation.split('\n'),
+                    width=self.wrapwidth,
+                )
             else:
-                lines = textwrap.wrap(translation, width=80)
-            for line in lines:
-                self._current_line += '%s\n' % polib.unescape(line)
-        else:
-            self._current_line += polib.unescape(translation)
+                lines = textwrap.wrap(
+                    translation,
+                    width=self.wrapwidth,
+                    break_long_words=False,
+                )
+            translation = '\n'.join(lines) + '\n'
+        self._current_line += polib.unescape(translation)
+
         self._current_msgid = ''
         self._current_msgctxt = None
 
@@ -222,7 +328,9 @@ class Po2Md:
         self._aimg_title_inside_current_msgid = False
 
     def _save_current_line(self):
-        self._outputlines.append(self._current_line.rstrip(' '))
+        # strip all spaces according to unicodedata database ignoring newlines,
+        # see https://docs.python.org/3/library/stdtypes.html#str.splitlines
+        self._outputlines.append(self._current_line.rstrip(' \v\x0b\f\x0c'))
         self._current_line = ''
 
     def enter_block(self, block, details):
@@ -236,8 +344,19 @@ class Po2Md:
             self._inside_pblock = True
         elif block.value == md4c.BlockType.CODE:
             self._inside_codeblock = True
+            indent = ''
+
+            if self._inside_liblock:
+                self._save_current_msgid()
+                if self._current_line:
+                    self._save_current_line()
+                indent += '   ' * len(self._current_list_type)
+
             if 'fence_char' in details:
-                self._current_line += '%s' % (details['fence_char']*3)
+                self._current_line += '{}{}'.format(
+                    indent,
+                    details['fence_char']*3,
+                )
             if details['lang']:
                 self._current_line += details['lang'][0][1]
             if 'fence_char' not in details:
@@ -247,33 +366,36 @@ class Po2Md:
         elif block.value == md4c.BlockType.H:
             self._current_line += '%s ' % ('#' * details['level'])
         elif block.value == md4c.BlockType.LI:
-            if self._current_ol_delimiters:
+            if self._current_list_type[-1][0] == 'ol':
                 # inside OL
-                if len(self._current_ol_delimiters) > 1:
+                if len(self._ol_marks) > 1:
                     self._save_current_msgid()
-                    if not self._current_ol_delimiters[-1][0]:
+                    if not self._ol_marks[-1][0]:
                         self._save_current_line()
-                self._current_ol_delimiters[-1][0] += 1
-                self._current_line += '%s%d%s ' % (
-                    '   ' * (len(self._current_ol_delimiters) - 1),
-                    self._current_ol_delimiters[-1][0],
-                    self._current_ol_delimiters[-1][1],
+                self._ol_marks[-1][0] += 1
+                self._current_line += '{}1{} '.format(
+                    '   ' * (len(self._ol_marks) - 1),
+                    self._ol_marks[-1][1],
                 )
+                self._current_list_type[-1][-1].append(False)
             else:
                 # inside UL
                 self._current_line += '{}{} '.format(
-                    '  ' * (len(self._ul_marks) - 1), self._ul_marks[-1],
+                    '   ' * (len(self._ul_marks) - 1), self._ul_marks[-1],
                 )
                 if details['is_task']:
                     self._current_line += '[%s] ' % details['task_mark']
+                self._current_list_type[-1][-1].append(details['is_task'])
             self._inside_liblock = True
         elif block.value == md4c.BlockType.UL:
             if len(self._ul_marks) > 0:
                 self._save_current_msgid()
                 self._save_current_line()
+            self._current_list_type.append(['ul', []])
             self._ul_marks.append(details['mark'])
         elif block.value == md4c.BlockType.OL:
-            self._current_ol_delimiters.append([0, details['mark_delimiter']])
+            self._current_list_type.append(['ol', []])
+            self._ol_marks.append([0, details['mark_delimiter']])
         elif block.value == md4c.BlockType.HR:
             self._current_line += '---'
             if not self._inside_liblock:
@@ -296,14 +418,27 @@ class Po2Md:
             if not self._inside_liblock:
                 self._save_current_line()
             self._inside_pblock = False
+            if self._inside_quoteblock:
+                self._current_line = '>'
+                self._save_current_line()
         elif block.value == md4c.BlockType.CODE:
             self._save_current_msgid()
             self._inside_codeblock = False
             self._inside_indented_codeblock = False
+
+            indent = ''
+            if self._inside_liblock:
+                indent += '   ' * len(self._current_list_type)
             if 'fence_char' in details:
-                self._current_line += '%s' % (details['fence_char']*3)
-            self._save_current_line()
+                if self._inside_liblock:
+                    self._save_current_line()
+                self._current_line += '{}{}'.format(
+                    indent,
+                    details['fence_char']*3,
+                )
             self._save_current_line()
+            if not self._inside_liblock:
+                self._save_current_line()
         elif block.value == md4c.BlockType.H:
             self._save_current_msgid()
             if not self._inside_quoteblock:
@@ -316,18 +451,21 @@ class Po2Md:
         elif block.value == md4c.BlockType.LI:
             self._save_current_msgid()
             self._inside_liblock = False
-            self._save_current_line()
+            if self._current_line:
+                self._save_current_line()
         elif block.value == md4c.BlockType.UL:
             self._ul_marks.pop()
+            self._current_list_type.pop()
             if self._inside_quoteblock:
                 self._current_line += '> '
-            if not self._ul_marks:
+            if not self._ul_marks and self._outputlines[-1]:
                 self._save_current_line()
         elif block.value == md4c.BlockType.OL:
-            self._current_ol_delimiters.pop()
+            self._ol_marks.pop()
+            self._current_list_type.pop()
             if self._inside_quoteblock:
                 self._current_line += '> '
-            if not self._current_ol_delimiters:
+            if not self._ol_marks and self._outputlines[-1]:
                 self._save_current_line()
         elif block.value in (md4c.BlockType.TH, md4c.BlockType.TD):
             self._save_current_msgid()
@@ -337,11 +475,17 @@ class Po2Md:
                 self._current_line += '|'
                 self._save_current_line()
         elif block.value == md4c.BlockType.THEAD:
-            self._current_line += '|'
-
             # build thead separator
-            thead_separator = '| '
-            _thead_split = re.split(r'[^\\](\|)', self._current_line)
+            thead_separator = ''
+            if self._inside_quoteblock:
+                _thead_split = re.split(r'[^\\](\|)', self._current_line)
+                self._current_line += '|'
+                thead_separator += '> '
+            else:
+                self._current_line += '|'
+                _thead_split = re.split(r'[^\\](\|)', self._current_line)
+            thead_separator += '| '
+
             for i, title in enumerate(_thead_split):
                 if (i % 2) != 0 or i > len(_thead_split) - 2:
                     continue
@@ -367,7 +511,9 @@ class Po2Md:
             self._current_line += '\n%s' % thead_separator
             self._save_current_line()
         elif block.value == md4c.BlockType.QUOTE:
-            self._current_line += '> '
+            if self._outputlines[-1] == '>':
+                self._outputlines.pop()
+            self._save_current_line()
         elif block.value == md4c.BlockType.TABLE:
             self._save_current_line()
         elif block.value == md4c.BlockType.DOC:
@@ -484,6 +630,11 @@ class Po2Md:
                     return
                 self._current_msgid += polib.escape(text)
             else:
+
+                if self._inside_liblock:
+                    indent = '   ' * len(self._current_list_type)
+                    if self._current_line[:len(indent)+1] != indent:
+                        self._current_line += indent
                 self._current_msgid += text
         else:
             self._process_command(text)
@@ -533,11 +684,22 @@ class Po2Md:
 
 
 def pofile_to_markdown(
-    filepath_or_content, pofiles, ignore=[],
-    save=None, md_encoding='utf-8', po_encoding=None, **kwargs,
+    filepath_or_content,
+    pofiles,
+    ignore=[],
+    save=None,
+    md_encoding='utf-8',
+    po_encoding=None,
+    command_aliases={},
+    wrapwidth=80,
+    **kwargs,
 ):
     r"""Translate Markdown content or a file using pofiles for message replacing.
 
+    This implementation reproduces the same valid Markdown output, given the
+    provided AST, with replaced translations, but doesn't rebuilds the same
+    input format as Markdown is just a subset of HTML.
+
     Args:
         filepath_or_content (str): Markdown filepath or content to translate.
         pofiles (str): Glob matching a set of pofiles from where to extract
@@ -553,13 +715,26 @@ def pofile_to_markdown(
             for each file, you must define it in the "Content-Type" field of
             each PO file metadata, in the form
             ``"Content-Type: text/plain; charset=<ENCODING>\n"``.
+        command_aliases (dict): Mapping of aliases to use custom mdpo command
+            names in comments. The ``mdpo-`` prefix in command names resolution
+            is optional. For example, if you want to use ``<!-- mdpo-on -->``
+            instead of ``<!-- mdpo-enable -->``, you can pass the dictionaries
+            ``{"mdpo-on": "mdpo-enable"}`` or ``{"mdpo-on": "enable"}`` to this
+            parameter.
+        wrapwidth (int): Maximum width used rendering the Markdown output.
 
     Returns:
         str: Markdown output file with translated content.
     """
     return Po2Md(
-        pofiles, ignore=ignore, po_encoding=po_encoding,
+        pofiles,
+        ignore=ignore,
+        po_encoding=po_encoding,
+        command_aliases=command_aliases,
+        wrapwidth=wrapwidth,
         **kwargs,
     ).translate(
-        filepath_or_content, save=save, md_encoding=md_encoding,
+        filepath_or_content,
+        save=save,
+        md_encoding=md_encoding,
     )
