@@ -5,16 +5,30 @@
 import argparse
 import sys
 
-from mdpo.cli import add_common_cli_arguments, parse_list_cli_argument
+from mdpo.cli import (
+    add_common_cli_first_arguments,
+    add_common_cli_latest_arguments,
+    parse_command_aliases_cli_argument,
+    parse_list_cli_argument,
+)
 from mdpo.po2md import pofile_to_markdown
 
 
-DESCRIPTION = 'Markdown files translator using pofiles as reference.'
+DESCRIPTION = (
+    'Markdown files translator using pofiles as reference.\n\n'
+    'This implementation reproduces the same valid Markdown output, given the'
+    " provided AST, with replaced translations, but doesn't rebuilds the same"
+    ' input format as Markdown is just a subset of HTML.'
+)
 
 
 def build_parser():
-    parser = argparse.ArgumentParser(description=DESCRIPTION, add_help=False)
-    add_common_cli_arguments(parser)
+    parser = argparse.ArgumentParser(
+        description=DESCRIPTION,
+        add_help=False,
+        formatter_class=argparse.RawDescriptionHelpFormatter,
+    )
+    add_common_cli_first_arguments(parser)
     parser.add_argument(
         'filepath_or_content', metavar='FILEPATH_OR_CONTENT',
         nargs='*',
@@ -37,6 +51,11 @@ def build_parser():
         help='Saves the output content in file whose path is'
              ' specified at this parameter.', metavar='PATH',
     )
+    parser.add_argument(
+        '-w', '--wrapwidth', dest='wrapwidth', default=80, type=int,
+        help='Maximum width rendering the Markdown output.',
+        metavar='N',
+    )
     parser.add_argument(
         '--md-encoding', dest='md_encoding', default='utf-8',
         help='Markdown content encoding.', metavar='<ENCODING>',
@@ -44,11 +63,12 @@ def build_parser():
     parser.add_argument(
         '--po-encoding', dest='po_encoding', default=None,
         help='PO files encoding. If you need different encodings for each'
-             ' file, you must define it in the "Content-Type" field of each'
+             ' file, you must define them in the "Content-Type" field of each'
              ' PO file metadata, in the form "Content-Type: text/plain;'
              ' charset=<ENCODING>\\n".',
         metavar='<ENCODING>',
     )
+    add_common_cli_latest_arguments(parser)
     return parser
 
 
@@ -66,6 +86,10 @@ def parse_options(args):
     if opts.ignore:
         opts.ignore = parse_list_cli_argument(opts.ignore)
 
+    opts.command_aliases = parse_command_aliases_cli_argument(
+        opts.command_aliases,
+    )
+
     return opts
 
 
@@ -77,6 +101,8 @@ def run(args=[]):
         ignore=opts.ignore, save=opts.save,
         md_encoding=opts.md_encoding,
         po_encoding=opts.po_encoding,
+        command_aliases=opts.command_aliases,
+        wrapwidth=opts.wrapwidth,
     )
 
     if not opts.quiet and not opts.save:
